package com.server;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.SQLException;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import shared.Commande;
import shared.CommandeEnCours_Thread;
import shared.ConnexionDB;
import shared.Message;
import shared.Plat;

public class Main_Server_Thread implements Runnable 
{

    public Socket_Client_Thread clients[];  
    public Socket_Cuisinier_Thread cuisiniers[]; 
    public ServerSocket server = null;  
    public ObjectInputStream streamIn  =  null;
    public ObjectOutputStream streamOut = null;
    public Thread       thread = null;  
    public int clientCount = 0, cuisiniersCount = 0, port; 
    public String URL_Mysql,user, password;
    public Server_Gui ui;  
    public ConnexionDB db;
    boolean exit;
    ThreadGroup rootThreadGroup = null;
    public ArrayList<Commande> listeCommandeDemandes = new ArrayList<Commande>();
    public ArrayList<Commande> listeCommandeEnCours = new ArrayList<Commande>();
    public ArrayList<Commande> listeCommandeTermines = new ArrayList<Commande>();
    
    public ArrayList<CommandeEnCours_Thread> listeCommandeEnCours_Thread = new ArrayList<CommandeEnCours_Thread>();
    

    public Main_Server_Thread(Server_Gui ui) {
        
        this.ui = ui;
        clients = new Socket_Client_Thread[50]; 
        cuisiniers = new Socket_Cuisinier_Thread[50]; 
        db = ui.MyConnexionDB;
        exit = false;
                
	  
	    try{  
                port = Integer.parseInt(ui.Port_jTextField.getText());
                URL_Mysql = ui.URL_Mysql_jTextField.getText();
                user = ui.userMysql_jTextField.getText();
                char[] PassCharArray = ui.Password_Mysql_jPassword.getPassword();
                password = String.valueOf(PassCharArray);
                
                try
                {
                    server = new ServerSocket(port);
                    port = server.getLocalPort();
                    ui.Log_jTextArea.append("\nle Serveur Socket est lancé. @IP : " + InetAddress.getLocalHost() + ", Port : " + server.getLocalPort());
                    start(); 
                }catch(IOException ioe){  
                    ui.Log_jTextArea.append("impossible d'écouter sur le port : " + port + "\nNouvelle tentative\n"); 
                    ui.RetryStart(0);
                   }
                
            }catch(NumberFormatException e)
            				{	
            					JOptionPane erreur = new JOptionPane();
                                                ImageIcon img = new ImageIcon(getClass().getResource("/images/erreur.png"));
                                                erreur.showMessageDialog(null, "Veuillez Spécifier un port valide", "Erreur", JOptionPane.ERROR_MESSAGE, img);
                                                ui.Port_jTextField.setText("20000");
                                                ui.ExistErreur = true;
                                        }
    }
    
        public Main_Server_Thread(Server_Gui frame, int Port){
       
        clients = new Socket_Client_Thread[50];
        cuisiniers = new Socket_Cuisinier_Thread[50];
        ui = frame;
        port = Port;
        
	try{  
	    server = new ServerSocket(port);
            port = server.getLocalPort();
	    ui.Log_jTextArea.append("le Serveur Socket est lancé. @IP : " + InetAddress.getLocalHost() + ", Port : " + server.getLocalPort());
	    start(); 
        }
	catch(IOException ioe){  
            ui.Log_jTextArea.append("\nimpossible d'écouter sur le port : " + port + ": " + ioe.getMessage()); 
	}
    }
    public void start(){  
    	if (thread == null)
        {
            
            thread = new Thread(this); 
	    thread.start();
	}
    }
    
    @SuppressWarnings("deprecation")
    public void stop()
    {  
        if (thread != null){  
            thread.stop(); 
	    thread = null;
	}
    }
    
    public void close() throws IOException 
    {  
    	if (server != null)    server.close();
        ui.Log_jTextArea.append("\nArret du Serveur d'ecoute -- DOWN \n\n");
        exit = true;
        this.stop();
    }
    
    public int findClient(int ID){  
    	for (int i = 0; i < clientCount; i++)
        {
        	if (clients[i].getID() == ID){
                    return i;
                }
	}
	return -1;
    }
    public int findClientByName(String Name)
    {
        for (int i = 0; i < clientCount; i++)
        {
        	if (clients[i].username.equals(Name))
                {
                    return i;
                }
	}
        return -1;
    }
            
    public int findCuisinier(int ID){  
    	for (int i = 0; i < cuisiniersCount; i++){
        	if (cuisiniers[i].getID() == ID){
                    return i;
                }
	}
	return -1;
    }
    
    @Override
    public void run()
    {  
	while (thread != null && !exit)
        {  
            try{  
		ui.Log_jTextArea.append("\nEn attente de client ...");
                LoadCommandsIntoArrays();
	        addThread(server.accept()); 
	    }
	    catch(Exception ioe){ 
                
                ui.Log_jTextArea.append("\nErreur Accept du serveur\n");
                System.out.println("Erreur Accept du serveur");
                ui.RetryStart(0);
	    }
        }
    }
    
     private void addThread(Socket socket) throws IOException, ClassNotFoundException 
     {  
	
        streamOut = new ObjectOutputStream(socket.getOutputStream());
        streamOut.flush();
        streamIn = new ObjectInputStream(socket.getInputStream());
        
        Message msg = (Message) streamIn.readObject();
        
        if(msg.type.equals("login") && msg.corps.equals("Serveur"))
        {
            if (clientCount < clients.length)
            {  
                ui.Log_jTextArea.append("\nClient accepté: " + socket);
                clients[clientCount] = new Socket_Client_Thread(this,socket,streamIn, streamOut);
                clients[clientCount].start();  
                clientCount++;
            }
            else
            {
                ui.Log_jTextArea.append("\nclient refusé: maximum " + clients.length + " atteint.");
            }
        }
        else if(msg.type.equals("login") && msg.corps.equals("Cuisinier"))
        {
            if (cuisiniersCount < cuisiniers.length)
            {  
                ui.Log_jTextArea.append("\nCuisinier accepté: " + socket);
                cuisiniers[cuisiniersCount] = new Socket_Cuisinier_Thread(this,socket,streamIn, streamOut);
                cuisiniers[cuisiniersCount].start();  
                cuisiniersCount++;
            }
            else
            {
                ui.Log_jTextArea.append("\nCuisinier refusé: maximum " + cuisiniers.length + " atteint.");
            }
        }
    }
     
    public void AnnounceCuisinier(String type, String emmeteur, String corps)
    {
        Message msg = new Message(type, emmeteur, corps, "MultiCast");
        for(int i = 0; i < cuisiniersCount; i++)
        {
            cuisiniers[i].send(msg);
        }
    }
    public void AnnounceClient(String type, String emmeteur, String corps)
    {
        Message msg = new Message(type, emmeteur, corps, "MultiCast");
        for(int i = 0; i < clientCount; i++)
        {
            clients[i].send(msg);
        }
    }
    
    public void SendClientList(String toWhom){
        for(int i = 0; i < clientCount; i++)
        {
            findClientThread(toWhom).send(new Message("newclient", "SERVER", clients[i].username, toWhom));
        }
    }
    
    public void SendCuisinierList(String toWhom){
        for(int i = 0; i < cuisiniersCount; i++)
        {
            findCuisinierThread(toWhom).send(new Message("newcuisinier", "SERVER", cuisiniers[i].username, toWhom));
        }
    }
    
    public Socket_Client_Thread findClientThread(String usr)
    {
        for(int i = 0; i < clientCount; i++)
        {
            if(clients[i].username.equals(usr)){
                return clients[i];
            }
        }
        return null;
    }
    
    public Socket_Cuisinier_Thread findCuisinierThread(String usr){
        for(int i = 0; i < cuisiniersCount; i++)
        {
            if(cuisiniers[i].username.equals(usr)){
                return cuisiniers[i];
            }
        }
        return null;
    }
	
    @SuppressWarnings("deprecation")
    public synchronized void removeClient(int ID)
    {  
        
        int pos = findClient(ID);
        if (pos >= 0){  
            Socket_Client_Thread toTerminate = clients[pos];
            ui.Log_jTextArea.append("\nSuppression de Thread du Client " + ID + " à " + pos);
	    if (pos < clientCount-1){
                for (int i = pos+1; i < clientCount; i++){
                    clients[i-1] = clients[i];
	        }
	    }
	    clientCount--;
	    try{  
	      	toTerminate.close(); 
	    }
	    catch(IOException ioe){  
	      	ui.Log_jTextArea.append("\nErreur Fermeture du thread: " + ioe); 
	    }
	    toTerminate.stop(); 
	}
    }
    
     @SuppressWarnings("deprecation")
    public synchronized void removeCuisinier(int ID)
    {  
        
        int pos = findCuisinier(ID);
        if (pos >= 0){  
            Socket_Cuisinier_Thread toTerminate = cuisiniers[pos];
            ui.Log_jTextArea.append("\nSuppression de Thread du Cuisinier " + ID + " à " + pos);
	    if (pos < cuisiniersCount-1)
            {
                for (int i = pos+1; i < cuisiniersCount; i++)
                {
                    cuisiniers[i-1] = cuisiniers[i];
	        }
	    }
	    cuisiniersCount--;
	    try{  
	      	toTerminate.close(); 
	    }
	    catch(IOException ioe){  
	      	ui.Log_jTextArea.append("\nErreur Fermeture du thread: " + ioe); 
	    }
	    toTerminate.stop(); 
	}
    }
    
    public void LoadCommandsIntoArrays()
    {
        listeCommandeDemandes = ui.MyConnexionDB.LoadListeCommandeByEtat(1);
        listeCommandeEnCours = ui.MyConnexionDB.LoadListeCommandeByEtat(2);
        listeCommandeTermines = ui.MyConnexionDB.LoadListeCommandeByEtat(3);
    }            
    public synchronized void handleClient(int ID, Message msg) throws SQLException
    {  
        if (msg.corps.equals(".bye"))
        {
            AnnounceClient("signout", "SERVER", msg.emmeteur);
            removeClient(ID); 
	}
	else
        {
            if(msg.type.equals("login"))
            {
                if(findClientThread(msg.emmeteur) == null)
                { 
                    if(ui.MyConnexionDB.checkLoginClient(msg.emmeteur, msg.corps))
                    {
                        clients[findClient(ID)].username = msg.emmeteur;
                        clients[findClient(ID)].send(new Message("login", "SERVER", "TRUE", msg.emmeteur));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        AnnounceClient("newclient", "SERVER", msg.emmeteur);
                        SendClientList(msg.emmeteur);
                    }
                    else
                    {
                        clients[findClient(ID)].send(new Message("login", "SERVER", "FALSE", msg.emmeteur)); 
                    }
                }
                else
                {
                    clients[findClient(ID)].send(new Message("login", "SERVER", "FALSE1", msg.emmeteur));            
                }
            }
            else if(msg.type.equals("ListPlat"))
            {
                ArrayList<Plat> listePlats = (ArrayList<Plat>) ui.MyConnexionDB.GetListPlatCuisinier();
                if(!listePlats.isEmpty())
                    {
                        clients[findClient(ID)].send(new Message("ListPlat", "SERVER", "TRUE", msg.emmeteur, listePlats));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        // plein
                    }
                    else
                    {// vide
                        clients[findClient(ID)].send(new Message("ListPlat", "SERVER", "FALSE", msg.emmeteur,listePlats)); 
                    }
            }
            else if(msg.type.equals("ListCommandesEnCours_Client"))
            {
                LoadCommandsIntoArrays();
                ArrayList<Commande> listeCommandeAll12 = new ArrayList<Commande>();
                ArrayList<Commande> listeCommandeByClient = new ArrayList<Commande>();
                listeCommandeAll12.addAll(listeCommandeDemandes);
                listeCommandeAll12.addAll(listeCommandeEnCours);
                
                for(int i=0;i<listeCommandeAll12.size();i++)
                {
                    Commande TempC = listeCommandeAll12.get(i);
                    if(TempC.getNom_client().equals(msg.emmeteur))
                    {
                        listeCommandeByClient.add(TempC);
                    }
                }
                if(!listeCommandeByClient.isEmpty())
                    {
                        Message newM = new Message("ListCommandesEnCours_Client", "SERVER", "TRUE", msg.emmeteur);
                        newM.setListeCommandes(listeCommandeByClient);
                        clients[findClient(ID)].send(newM);   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        
                    }
                    else
                    {// vide
                        clients[findClient(ID)].send(new Message("ListCommandesEnCours_Client", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            else if(msg.type.equals("ListCommandeTermines_Client"))
            {
                LoadCommandsIntoArrays();
                ArrayList<Commande> listeCommandeByClient = new ArrayList<Commande>();
                
                for(int i=0;i<listeCommandeTermines.size();i++)
                {
                    Commande TempC = listeCommandeTermines.get(i);
                    if(TempC.getNom_client().equals(msg.emmeteur))
                    {
                        listeCommandeByClient.add(TempC);
                    }
                }
                if(!listeCommandeByClient.isEmpty())
                    {
                        Message newM = new Message("ListCommandeTermines_Client", "SERVER", "TRUE", msg.emmeteur);
                        newM.setListeCommandes(listeCommandeByClient);
                        clients[findClient(ID)].send(newM);   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        
                    }
                    else
                    {// vide
                        clients[findClient(ID)].send(new Message("ListCommandeTermines_Client", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            else if(msg.type.equals("CommanderPlat"))
            {
                    if(ui.MyConnexionDB.CommanderPlatClient(msg.mycommande))
                    { 
                        LoadCommandsIntoArrays();
                        clients[findClient(ID)].send(new Message("CommanderPlat", "SERVER", "TRUE", msg.emmeteur,msg.mycommande));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        
                        for(int i = 0; i < cuisiniersCount; i++)
                        {
                            cuisiniers[i].send(new Message("NewIncommingCommande", "SERVER", "TRUE", "MultiCast", msg.mycommande));
                        }
                    }
                    else
                    {
                        clients[findClient(ID)].send(new Message("CommanderPlat", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            else if(msg.type.equals("message"))
            {
                if(msg.recepteur.equals("MultiCast"))
                {
                    AnnounceClient("message", msg.emmeteur, msg.corps);
                }
            }
        }
    }
    
    Thread getThread( final String name ) 
    {
        if ( name == null )
        {
            throw new NullPointerException( "Null name" );
        }    
        final Thread[] threads = getAllThreads( );
        for ( Thread threadX : threads )
        {
            if ( threadX.getName( ).equals( name ) )
            {
                return threadX;
            } 
        }
        return null;
    }
    Thread[] getAllThreads( ) 
    {
        final ThreadGroup root = getRootThreadGroup( );
        final ThreadMXBean thbean = ManagementFactory.getThreadMXBean( );
        int nAlloc = thbean.getThreadCount( );
        int n = 0;
        Thread[] threads;
        do {
            nAlloc *= 2;
            threads = new Thread[ nAlloc ];
            n = root.enumerate( threads, true );
        } while ( n == nAlloc );
        return java.util.Arrays.copyOf( threads, n );
    }
    
 
    ThreadGroup getRootThreadGroup( ) 
    {
        if ( rootThreadGroup != null )
            return rootThreadGroup;
        ThreadGroup tg = Thread.currentThread( ).getThreadGroup( );
        ThreadGroup ptg;
        while ( (ptg = tg.getParent( )) != null )
            tg = ptg;
        return tg;
    }
    
    public synchronized void handleCuisinier(int ID, Message msg) throws SQLException
    {  
	if (msg.corps.equals(".bye"))
        {
            AnnounceCuisinier("signout", "SERVER", msg.emmeteur);
            removeCuisinier(ID); 
	}
	else
        {
            if(msg.type.equals("login"))
            {
                if(findCuisinierThread(msg.emmeteur) == null)
                {
                    
                    if(ui.MyConnexionDB.checkLoginCuisinier(msg.emmeteur, msg.corps))
                    {
                        cuisiniers[findCuisinier(ID)].username = msg.emmeteur;
                        cuisiniers[findCuisinier(ID)].send(new Message("login", "SERVER", "TRUE", msg.emmeteur));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        AnnounceCuisinier("newcuisinier", "SERVER", msg.emmeteur);
                        SendCuisinierList(msg.emmeteur);
                    }
                    else
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("login", "SERVER", "FALSE", msg.emmeteur)); 
                    }
                }
                else
                {
                    cuisiniers[findCuisinier(ID)].send(new Message("login", "SERVER", "FALSE1", msg.emmeteur));            
                    
                }
            }
            else if(msg.type.equals("message"))
            {
                if(msg.recepteur.equals("MultiCast"))
                {
                    AnnounceCuisinier("message", msg.emmeteur, msg.corps);
                }/*
                else{ // message perso
                    findUserThread(msg.recepteur).send(new Message(msg.type, msg.emmeteur, msg.corps, msg.recepteur));
                    clients[findClient(ID)].send(new Message(msg.type, msg.emmeteur, msg.corps, msg.recepteur));
                }*/
            }
            else if(msg.type.equals("AjoutPlat"))
            {
                if(ui.MyConnexionDB.AjoutPlatCuisinier(msg.plat))
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("AjoutPlat", "SERVER", "TRUE", msg.emmeteur, msg.plat));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        
                    }
                    else
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("AjoutPlat", "SERVER", "FALSE", msg.emmeteur,msg.plat)); 
                    }
            }
            else if(msg.type.equals("SupprimerPlat"))
            {
                    ArrayList<Plat> listePlats = ui.MyConnexionDB.SupprPlatCuisinier(msg.plat);
                    if(listePlats != null)
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("SupprimerPlat", "SERVER", "TRUE", msg.emmeteur, listePlats));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                    }
                    else
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("SupprimerPlat", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            
            else if(msg.type.equals("ModifierPlat"))
            {
                ArrayList<Plat> listePlats = ui.MyConnexionDB.ModifPlatCuisinier(msg);
                    if(listePlats != null)
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("ModifierPlat", "SERVER", "TRUE", msg.emmeteur, listePlats));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                    }
                    else
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("ModifierPlat", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            else if(msg.type.equals("ListPlat"))
            {
                ArrayList<Plat> listePlats = (ArrayList<Plat>) ui.MyConnexionDB.GetListPlatCuisinier();
                if(!listePlats.isEmpty())
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("ListPlat", "SERVER", "TRUE", msg.emmeteur, listePlats));   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        // plein
                    }
                    else
                    {// vide
                        cuisiniers[findCuisinier(ID)].send(new Message("ListPlat", "SERVER", "FALSE", msg.emmeteur,listePlats)); 
                    }
            }
            else if(msg.type.equals("ListCommandesDemandes"))
            {
                LoadCommandsIntoArrays();
                if(!listeCommandeDemandes.isEmpty())
                    {
                        Message newM = new Message("ListCommandesDemandes", "SERVER", "TRUE", msg.emmeteur);
                        newM.setListeCommandes(listeCommandeDemandes);
                        cuisiniers[findCuisinier(ID)].send(newM);   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        System.out.println(ui.MyConnexionDB.LoadListeCommandeByEtat(1).toString());
                    }
                    else
                    {// vide
                        cuisiniers[findCuisinier(ID)].send(new Message("ListCommandesDemandes", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            else if(msg.type.equals("ListCommandeTermines"))
            {
                LoadCommandsIntoArrays();
                if(!listeCommandeTermines.isEmpty())
                    {
                        Message newM = new Message("ListCommandeTermines", "SERVER", "TRUE", msg.emmeteur);
                        newM.setListeCommandes(listeCommandeTermines);
                        cuisiniers[findCuisinier(ID)].send(newM);   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        //System.out.println(ui.MyConnexionDB.LoadListeCommandeByEtat(1).toString());
                    }
                    else
                    {// vide
                        cuisiniers[findCuisinier(ID)].send(new Message("ListCommandeTermines", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            //CommencerPreparerPlat
            else if(msg.type.equals("CommencerPreparerPlat"))
            {
                
                if(db.CommencerPreparerCommandeMoveTo2(msg.mycommande))
                    {
                        LoadCommandsIntoArrays();
                        cuisiniers[findCuisinier(ID)].send(new Message("CommencerPreparerPlat", "SERVER", "TRUE", msg.emmeteur));
                        
                        for(int i = 0; i < cuisiniersCount; i++)
                        {
                            
                            Message newM = new Message("ListCommandesDemandes", "SERVER", "TRUE", msg.emmeteur);
                            newM.setListeCommandes(listeCommandeDemandes);
                            cuisiniers[i].send(newM);
                            LoadCommandsIntoArrays();
                            Message MessageCommandesEnCours = new Message("ListCommandesEnCours", "SERVER", "TRUE", msg.emmeteur);
                            MessageCommandesEnCours.setListeCommandes(listeCommandeEnCours);
                            cuisiniers[i].send(MessageCommandesEnCours);    
                        }
                        //ProgressBar Operations
                        LoadCommandsIntoArrays();
                        CommandeEnCours_Thread newCommandThread = new CommandeEnCours_Thread(msg.mycommande, ui, ID);
                        newCommandThread.start();
                    }
                    else
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("CommencerPreparerPlat", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            else if(msg.type.equals("SpecificCommandesEnCoursDetails"))
            {
                        listeCommandeEnCours_Thread = db.getListeCommandeEnCours_Thread();
                        for(int i=0;i<listeCommandeEnCours_Thread.size();i++)
                        {
                            listeCommandeEnCours_Thread.get(i).setUi(ui);
                            String ThreadName = "Thread Commande n° "+msg.mycommande.getID_Commande();
                            if(ThreadName.equals(listeCommandeEnCours_Thread.get(i).getThreadName()))
                            {
                                msg.mycommande.setThreadName(listeCommandeEnCours_Thread.get(i).getThreadName());
                                msg.mycommande.setTimePreparationinMiliSecond(listeCommandeEnCours_Thread.get(i).getTimePreparationinMiliSecond());
                                msg.mycommande.setProgressValue(listeCommandeEnCours_Thread.get(i).getProgressValue());
                                msg.mycommande.setTempsEcoule(listeCommandeEnCours_Thread.get(i).getTempsEcoule());
                                msg.mycommande.setClackRate(listeCommandeEnCours_Thread.get(i).getClackRate());
                                
                                // THread Search
                                Thread Temp_Thread = getThread(ThreadName);
                                if(Temp_Thread == null)
                                {
                                    listeCommandeEnCours_Thread.get(i).start();
                                }
                            }
                        }
                //System.out.println(msg.mycommande);
                cuisiniers[findCuisinier(ID)].send(new Message("SpecificCommandesEnCoursDetails", "SERVER", "TRUE", msg.emmeteur, msg.mycommande));
            }
            else if(msg.type.equals("ListCommandesEnCours"))
            {
                LoadCommandsIntoArrays();
                //listeCommandeEnCours_Thread;
                    if(!listeCommandeEnCours.isEmpty())
                    {
                        Message MessageCommandesDemandes = new Message("ListCommandesDemandes", "SERVER", "TRUE", msg.emmeteur);
                        MessageCommandesDemandes.setListeCommandes(listeCommandeDemandes);
                        cuisiniers[findCuisinier(ID)].send(MessageCommandesDemandes);   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                        
                        Message MessageCommandesEnCours = new Message("ListCommandesEnCours", "SERVER", "TRUE", msg.emmeteur);
                        MessageCommandesEnCours.setListeCommandes(listeCommandeEnCours);
                        cuisiniers[findCuisinier(ID)].send(MessageCommandesEnCours);   // System.out.println("envoi : "+new Message("login", "SERVER", "TRUE", msg.emmeteur).toString());
                    }
                    else
                    {
                        cuisiniers[findCuisinier(ID)].send(new Message("ListCommandesEnCours", "SERVER", "FALSE", msg.emmeteur)); 
                    }
            }
            else if(msg.type.equals("inscription"))
            {
                if(findCuisinierThread(msg.emmeteur) == null)
                {
                    if(!db.CuisinierExists(msg.emmeteur))
                    {
                        db.addCuisinier(msg.emmeteur, msg.corps);
                        cuisiniers[findCuisinier(ID)].username = msg.emmeteur;
                        cuisiniers[findCuisinier(ID)].send(new Message("inscription", "SERVER", "TRUE", msg.emmeteur));
                        cuisiniers[findCuisinier(ID)].send(new Message("login", "SERVER", "TRUE", msg.emmeteur));
                        AnnounceCuisinier("newcuisinier", "SERVER", msg.emmeteur);
                        SendCuisinierList(msg.emmeteur);
                    }
                    else{
                        cuisiniers[findCuisinier(ID)].send(new Message("inscription", "SERVER", "FALSE", msg.emmeteur));
                    }
                }
                else{
                    cuisiniers[findCuisinier(ID)].send(new Message("inscription", "SERVER", "FALSE", msg.emmeteur));
                }
            }
	}
    }
    
}
